package com.simonstl.fragment;
import org.xml.sax.helpers.AttributesImpl;
import org.xml.sax.*;
import org.xml.sax.helpers.AttributesImpl;
import org.xml.sax.helpers.DefaultHandler;
import org.xml.sax.XMLFilter;
import org.xml.sax.helpers.XMLFilterImpl;

/*
The contents of this file are subject to the Mozilla Public License  Version 1.1 (the "License"); you may not use this file except in  compliance with the License. You may obtain a copy of the License at 
http://www.mozilla.org/MPL/ 
Software distributed under the License is distributed on an "AS IS"  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the  License for the specific language governing rights and limitations under  the License. 

The Original Code is available at  http://simonstl.com/projects/fragment/original. 

The Initial Developer of the Original Code is Simon St.Laurent. Portions  created by Simon St.Laurent are Copyright (C) 2001. All Rights Reserved. 

Contributor(s): 
*/

/**
* <p>The Element class stores information about elements (and perhaps someday attributes)
* in a format that supports namespaces, allows the preservation of prefix information,
* and supports matching on namespace URIs and localNames.</p>
*
* <p>I may eventually add support for attributes, and also for wildcard matching on nsURI
* and/or localName.</p>
*
* <p>Version 0.06 extends DocComponent rather than trying to handle all cases itself.</p>
*
* <p>Version 0.05 adds support for <code>beforeInside</code> and <code>afterInside</code> attributes, 
* allowing rules to add content inside of the element they create.</p>
*
* <p>Version 0.04 changed support for skipping, allowing for the <code>chars</code> 
* element to operate when character output is desired.  It also now calls the startChars() 
* and endChars() methods of {@link com.simonstl.fragment.FragmentFilter FragmentFilter} to 
* give these characters their own place in the stack.  Finally, it now supports 
* <code>before</code> and <code>after</code> content.</p>
*
* <p>Version 0.03 added properties for content and attributes.  It also added the eventElement()
* method, which allows Element objects to write out their own SAX events.  This should make
* it easier to manage configuration in the future.</p>
*
* <p>version 0.02 added the getKey() method, so that Elements could define their own keys.
* This should permit future versions to support more complex matching than just nsURI>localName.
* (Making that work will involve revisions to matchElement (String match) as well.)</p>
*
* @version 0.06 1 July 2001
* @author Simon St.Laurent
*/

public class Element extends DocComponent {
    private AttributesImpl atts=null;
    private String beforeInside=null;
    private String afterInside=null;

public Element (String newNsURI, String newLocalName) {
    nsURI=newNsURI;
    localName=newLocalName;
    prefix="";
}

public Element (String newNsURI, String newLocalName, String newPrefix) {
    nsURI=newNsURI;
    localName=newLocalName;
    prefix=newPrefix;
}

public Element (String newNsURI, String newLocalName, String newPrefix, String newContent) {
    nsURI=newNsURI;
    localName=newLocalName;
    prefix=newPrefix;
    content=newContent;
}

public Element (Element oldElement) {
    nsURI=new String (oldElement.getNsURI());
    localName=new String (oldElement.getLocalName());
    prefix=new String (oldElement.getPrefix());
    before=oldElement.getBefore();
    after=oldElement.getAfter();
    beforeInside=oldElement.getBeforeInside();
    afterInside=oldElement.getAfterInside();
}


public boolean matchComponent (String match) {
    return (match.equals(nsURI+">"+localName));
}

public String getKey() {
    return nsURI+">"+localName;
}

public void setAtts(AttributesImpl newAtts) {
      atts=newAtts;
}

public AttributesImpl getAtts() {
      return atts;
}


public void setBeforeInside(String newBeforeInside) {
      beforeInside=newBeforeInside;
}

public String getBeforeInside() {
      return beforeInside;
}
public void setAfterInside(String newAfterInside) {
      after=newAfterInside;
}

public String getAfterInside() {
      return afterInside;
}


public void eventComponent(FragmentFilter handler) throws SAXException {
      String QName;
//System.out.println("skip status: "+skip);
      if (atts==null) {
        atts=new AttributesImpl();
      }

       if (!(prefix.equals(""))) {
        QName=prefix+":"+localName;
       } else {
        QName=localName;
       }
//if there's content for before the element, present it
      if (before!=null) {
        handler.startChars(); 
        handler.characters(before.toCharArray(), 0, before.length());
        handler.endChars(); 
      }

//if the element has a name, start it.
      if (!(( nsURI.equals("") ) && ( localName.equals("") ) && ( QName.equals("") ))) {
        handler.startElement(nsURI, localName, QName, atts);
//System.out.println("starting element " + QName);
      } else {
         if (!(content.equals(""))) {handler.startChars();}
      }

//if there's content for after the start tag but before the content, present it

      if (beforeInside!=null) {
        handler.startChars(); 
        handler.characters(beforeInside.toCharArray(), 0, beforeInside.length());
        handler.endChars(); 
      }

//if the element has content, present it
      if (!(content.equals(""))) {
        handler.characters(content.toCharArray(), 0, content.length());
//System.out.println("content " + content);
      }

//if there's content for after the content but before the end tag, present it

      if (afterInside!=null) {
        handler.startChars(); 
        handler.characters(afterInside.toCharArray(), 0, afterInside.length());
        handler.endChars(); 
      }
//end the element
      if (!(( nsURI.equals("") ) && ( localName.equals("") ) && ( QName.equals("") ))) {
        handler.endElement(nsURI, localName, QName);
//System.out.println("ending element " + QName);
      } else {
         if (!(content.equals(""))) {handler.endChars();}
      }
//if there's content for after the element, present it
      if (after!=null) {
        handler.startChars(); 
        handler.characters(after.toCharArray(), 0, after.length());
        handler.endChars(); 
      }

}


public void print() {
System.out.println("result: "+prefix+"("+nsURI+"):"+localName+" contains: " +content);
}

public String toString() {
   return (prefix+"("+nsURI+"):"+localName);
}

}